@@ -14,8 +14,8 @@ $LYNX = "NO";
 $GREP = "/usr/bin/grep";
 $RECODE = "NO"; #"/usr/bin/recode"; -- dont use recode, it is not widely available
 # Directories
-$BANKDATADIR="/home/stimming/usr/share/ktoblzcheck";
-$prefix="/home/stimming/usr";
+$BANKDATADIR="${pkgdatadir}";
+$prefix="/usr/local";
 $datadir="${prefix}/share";
 $pkgdatadir="$datadir/ktoblzcheck";
 
