@@ -1,6 +1,6 @@
 /***************************************************************************
                              -------------------
-    cvs         : $Id: algorithms.cc,v 1.11 2005/11/16 12:19:16 cstim Exp $
+    cvs         : $Id: algorithms.cc,v 1.17 2007/02/17 09:52:05 cstim Exp $
     begin       : Sat Aug 10 2002
     copyright   : (C) 2002 by Fabian Kaiser
     email       : fabian@openhbci.de
@@ -28,6 +28,7 @@
 # include "config.h"
 #endif
 #include "algorithms.h"
+#include "binreloc.h"
 #include <fstream>
 #include <iostream>
 #include <assert.h>
@@ -124,7 +125,9 @@ using namespace std;
 
 
 // ////////////////////
-
+#if 0
+// Unused because the lookup at runtime (below in
+// accnum_get_pkgdatadir) is much more efficient
 std::string accnum_getRegKey(const char *value_name) 
 {
 #if OS_WIN32
@@ -180,6 +183,74 @@ std::string accnum_getRegKey(const char *value_name)
    return "";
 #endif // OS_WIN32
 }
+#endif // null
+
+
+static std::string accnum_get_prefix()
+{
+#ifdef OS_WIN32
+   // Partial implementation of 
+   // http://developer.gnome.org/doc/API/2.0/glib/glib-Windows-Compatibility-Functions.html#g-win32-get-package-installation-directory
+   char ch_fn[MAX_PATH], *p;
+
+   // From the docs: Retrieves the fully-qualified path for the
+   // file that contains the specified module. If the first
+   // parameter is NULL, GetModuleFileName retrieves the path of
+   // the executable file of the current process.
+   if (!GetModuleFileName(NULL, ch_fn, MAX_PATH)) return "";
+
+   if ((p = strrchr(ch_fn, '\\')) != NULL)
+      *p = '\0';
+
+   p = strrchr(ch_fn, '\\');
+   if (p && (_stricmp(p+1, "bin") == 0 ||
+	     _stricmp(p+1, "lib") == 0))
+      *p = '\0';
+
+   return std::string(ch_fn);
+#else // OS_WIN32
+   return "";
+#endif // OS_WIN32
+}
+std::string algorithms_get_bankdata_dir()
+{
+   std::string pkgdatadir;
+#ifdef OS_WIN32
+   std::string prefix(accnum_get_prefix());
+   pkgdatadir = prefix + "\\share\\ktoblzcheck";
+#else // OS_WIN32
+   char *datadir = br_find_data_dir(0);
+   if (datadir)
+   {
+      pkgdatadir = std::string(datadir) + std::string("/ktoblzcheck");
+      free (datadir);
+   } else
+   {
+      pkgdatadir = BANKDATA_PATH;
+   }
+#endif // OS_WIN32
+   return pkgdatadir;
+}
+
+
+static bool ktoblzcheck_binreloc_initialized = false;
+bool algorithms_init_binreloc()
+{
+   if (!ktoblzcheck_binreloc_initialized)
+   {
+      BrInitError br_error;
+      
+      /* Init binreloc. Note: It is not totally clear whether the correct
+	 function might still be br_init() instead of br_init_lib(). */
+      if (!br_init_lib(&br_error)) {
+	 /* DBG_INFO(GWEN_LOGDOMAIN, "Error on br_init: %d\n", br_error); */
+	 ktoblzcheck_binreloc_initialized = false;
+      }
+      else
+	 ktoblzcheck_binreloc_initialized = true;
+   }
+   return ktoblzcheck_binreloc_initialized;
+}
 
 // ////////////////////
 
@@ -187,19 +258,7 @@ AccountNumberCheck::Result
 algo01(int modulus, int weight[10], bool crossfoot, 
 	   int checkIndex, int accountId[10]) {
 
-  int result = algo02(modulus, weight, crossfoot, accountId);
-
-  // compare the result with the real check number
-  if (accountId[checkIndex - 1] == result)
-	return AccountNumberCheck::OK;
-  else 
-	return AccountNumberCheck::ERROR;
-}
-
-// will be removed!!!
-int algo02(int modulus, int weight[10], bool crossfoot, int accountId[10]) {
   int result = algo03(modulus, weight, crossfoot, accountId, 0, 9);
-
   //  cout << "res%mod: " << result << endl;
   // no rest after division by modulus (in algo03)?
   // then the the result will be 0!
@@ -209,7 +268,40 @@ int algo02(int modulus, int weight[10], bool crossfoot, int accountId[10]) {
   // and calc the check number
   result = (modulus - result) % 10;
 
-  return result;
+  // compare the result with the real check number
+  if (accountId[checkIndex - 1] == result)
+	return AccountNumberCheck::OK;
+  else 
+	return AccountNumberCheck::ERROR;
+}
+
+// New algo02 for Pruefmethode 02 and similar methods
+// 04, 07, 14, 58, 85/a, A2/v2, B2/v1, B4/v2, C3/v2, C4/v2
+// For all of these methods modulus is usually 11 
+AccountNumberCheck::Result 
+algo02(int modulus, int weight[10], int checkIndex, int accountId[10],
+	int startAdd, int stopAdd) {
+  int res[10];
+
+  // mult the weight with the account id
+  multArray(accountId, weight, res);
+
+  // add all values, get modulus
+  int result = add(res, startAdd, stopAdd);
+  result %= modulus;
+
+  if (result!=0) {
+    // multi-digit check-numbers are not allowed
+    if (modulus-result >= 10)
+      return AccountNumberCheck::ERROR;
+    result = modulus-result;
+  }
+
+  // compare the result with the real check number
+  if (accountId[checkIndex - 1] == result)
+	return AccountNumberCheck::OK;
+  else 
+	return AccountNumberCheck::ERROR;
 }
 
 int algo03(int modulus, int weight[10], bool crossfoot, int accountId[10], 
@@ -593,7 +685,7 @@ void crossFoot(int *source) {
 
 int add(const int *source, int start, int stop) {
     const int *sp = source + start;
-    switch (stop-start) {
+    switch (stop - start) {
     case 0:
 	return sp[0];
     case 1:
@@ -621,6 +713,7 @@ int add(const int *source, int start, int stop) {
 	    + sp[5] + sp[6] + sp[7] + sp[8] + sp[9];
     default:
 	assert(0);
+	return 0; // never reached
     }
 }
 
