@@ -1,6 +1,6 @@
 /***************************************************************************
                              -------------------
-    cvs         : $Id: iban.cc,v 1.6 2005/04/20 11:03:32 cstim Exp $
+    cvs         : $Id: iban.cc,v 1.9 2007/02/17 09:52:05 cstim Exp $
     begin       : Tue Apr 19 2005
     copyright   : (C) 2005 by Andreas Degert (some parts Gerhard Gappmeier)
     email       : ad@papyrus-gmbh.de
@@ -291,10 +291,9 @@ IbanCheck::IbanCheck(const std::string& filename)
 {
   std::string fname = filename;
   if (fname.empty()) {
-    std::string registry_path = accnum_getRegKey("datadir");
-    std::string data_path = BANKDATA_PATH;
-    fname = (registry_path.empty() ? data_path : registry_path)
-      + 
+    algorithms_init_binreloc();
+    std::string datadir = algorithms_get_bankdata_dir();
+    fname = datadir + 
 #if OS_WIN32
       "\\"
 #else
