@@ -66,15 +66,16 @@ AccountNumberCheck::Result method_01(int *account, int *weight) {
 }
 AccountNumberCheck::Result method_02(int *account, int *weight) {
     number2Array("2987654320", weight);
-    return algo01(11, weight, false, 10, account);
+    return algo02(11, weight, 10, account, 0, 8);
 }
 AccountNumberCheck::Result method_03(int *account, int *weight) {
     number2Array("2121212120", weight);
     return algo01(10, weight, false, 10, account);	
 }
+// this is method "02" using modified weight
 AccountNumberCheck::Result method_04(int *account, int *weight) {
     number2Array("4327654320", weight);
-    return algo01(11, weight, false, 10, account);
+    return algo02(11, weight, 10, account, 0, 8);
 }
 AccountNumberCheck::Result method_05(int *account, int *weight) {
     number2Array("1371371370", weight);
@@ -84,10 +85,11 @@ AccountNumberCheck::Result method_06(int *account, int *weight) {
     number2Array("4327654320", weight);
     return algo01(11, weight, false, 10, account);
 }
+// this is method "02" using modified weight
 AccountNumberCheck::Result method_07(int *account, int *weight) {
-    number2Array("987654320", weight);
+    number2Array("0987654320", weight);
     weight[0] = 10;
-    return algo01(11, weight, false, 10, account);
+    return algo02(11, weight, 10, account, 0, 8);
 }
 AccountNumberCheck::Result method_08(int *account, int *weight) {
     if (atol(array2Number(account).c_str()) < 60000)
@@ -138,9 +140,10 @@ AccountNumberCheck::Result method_13(int *account, int *weight) {
     } 
     return AccountNumberCheck::OK;
 }
+// this is method "02" using modified weight
 AccountNumberCheck::Result method_14(int *account, int *weight) {
     number2Array("0007654320", weight);
-    return algo01(11, weight, false, 10, account);
+    return algo02(11, weight, 10, account, 3, 8);
 }
 AccountNumberCheck::Result method_15(int *account, int *weight) {
     number2Array("54320", weight);
@@ -527,24 +530,55 @@ AccountNumberCheck::Result method_56(int *account, int *weight) {
 	return AccountNumberCheck::OK;
     return AccountNumberCheck::ERROR;
 }
+// Modified Method 57, valid from December, 4th 2006
 AccountNumberCheck::Result method_57(int *account, int *weight) {
-    // validated with 8 test accountIDs (4 with + 4 w/o checksum)
-    // many numbers do not have a checksum:
     int firstTwo = atoi(array2Number(account).substr(0, 2).c_str());
     int firstSix = atoi(array2Number(account).substr(0, 6).c_str());
-    if ((51 > firstTwo) || (91 == firstTwo) || (95 < firstTwo) ||
-	(777777 == firstSix) || (888888 == firstSix))
-	return AccountNumberCheck::OK;
-
-    number2Array("1212121210", weight);
-    return algo01(10, weight, true, 10, account);
+    if(firstTwo == 0)	return AccountNumberCheck::ERROR;
+    if(	firstTwo >= 64 && firstTwo <= 66 ||
+	firstTwo >= 75 && firstTwo <= 82 ||
+	firstTwo == 51 || firstTwo == 55 || firstTwo == 61 ||
+	firstTwo == 70 || firstTwo == 73 || firstTwo == 88 ||
+	firstTwo == 94 || firstTwo == 95 ) {			// Variante 1
+	if ((777777 == firstSix) || (888888 == firstSix))	// Exception for variant 1
+	  return AccountNumberCheck::OK;
+	number2Array("1212121210", weight);
+	return algo01(10, weight, true, 10, account);	
+    } else if (	firstTwo >= 32 && firstTwo <= 39 ||
+	firstTwo >= 41 && firstTwo <= 49 ||
+	firstTwo >= 52 && firstTwo <= 54 ||
+	firstTwo >= 56 && firstTwo <= 60 ||
+	firstTwo >= 67 && firstTwo <= 69 ||
+	firstTwo >= 83 && firstTwo <= 87 ||
+	firstTwo >= 96 && firstTwo <= 98 ||
+	firstTwo == 62 || firstTwo == 63 || firstTwo == 71 || 
+	firstTwo == 72 || firstTwo == 74 || firstTwo == 89 ||
+	firstTwo == 90 || firstTwo == 92 || firstTwo == 93 ) {	// Variante 2
+	number2Array("1201212121", weight);
+	return algo01(10, weight, true, 3, account);
+    } else if (	firstTwo == 40 || firstTwo == 50 ||
+	firstTwo == 91 || firstTwo == 99 ) {			// Variante 3
+	return AccountNumberCheck::OK;		// method 09
+    } else if (	firstTwo >= 01 && firstTwo <= 31) {		// Variante 4
+	int threeAndFour = atoi(array2Number(account).substr(2, 2).c_str());
+	if ( firstSix == 18512 && account[6] == 5 && account[7] == 4 &&
+	account[8] == 3 && account[9] == 4 )			// Exception for "0185125434"
+	  return AccountNumberCheck::OK;
+	if ( threeAndFour >= 1 && threeAndFour <= 12 && atoi(array2Number(account).substr(6, 3).c_str()) < 500 )
+	  return AccountNumberCheck::OK;
+	else
+	  return AccountNumberCheck::ERROR;
+    }
+    return AccountNumberCheck::ERROR;
 }
 // Added by Jens Gecius, validated with four test accountIDs
 AccountNumberCheck::Result method_58(int *account, int *weight) {
     if (0 == atoi(array2Number(account).substr(0,5).c_str()))
 	return AccountNumberCheck::ERROR;
+
+    // this is method "02" using modified weight
     number2Array("0000654320", weight);
-    return algo01(11, weight, false, 10, account);
+    return algo02(11, weight, 10, account, 4, 8);
 }
 AccountNumberCheck::Result method_59(int *account, int *weight) {
     // ids less than 9 digits can not be checked
@@ -856,9 +890,10 @@ AccountNumberCheck::Result method_84(int *account, int *weight) {
     return algo01(7, weight, false, 10, account);
 }
 AccountNumberCheck::Result method_85(int *account, int *weight) {
+    // exception: (method "02" using modified weight)
     if (9 == account[2] && 9 == account[3]) {
 	number2Array("0087654320", weight);
-	return algo01(11, weight, false, 10, account);
+	return algo02(11, weight, 10, account, 2, 8);
     }
 
     // var a:
@@ -1096,11 +1131,10 @@ AccountNumberCheck::Result method_A1(int *account, int *weight) {
 }
 // Added by Jens Gecius, validated with six test accountIDs (4 positives, 2 negatives)
 AccountNumberCheck::Result method_A2(int *account, int *weight) {
-    number2Array("2121212120", weight);
+    number2Array("2121212120", weight);		// variant 1
     if (AccountNumberCheck::OK == algo01(10, weight, true, 10, account))
 	return AccountNumberCheck::OK;
-    number2Array("4327654320", weight);
-    return algo01(11, weight, false, 10, account);
+    return method_04(account,weight);		// variant 2
 }
 // Added by Jens Gecius, NOT validated as no bank seems to use it...
 AccountNumberCheck::Result method_A3(int *account, int *weight) {
@@ -1212,23 +1246,12 @@ AccountNumberCheck::Result method_B1(int *account, int *weight) {
     // variant 2 (method "01")
 }
 // Added by Jens Gecius (new Method as of September 6th, 2004)
-// Checked with Bundesbank-Testnumbers (one of them failed the test due to deeper
-// problem with algo02, where the result for method 02 is wrongly calculated!!
-// result shows account is valid although it is not. I don't know about deeper
-// implications of a change of that line:
-//   result = (modulus - result) % 10;
-// to
-//   result = (modulus - result)
-// as per method "02" with modulus 11 and test-result "10", it shows an invalid account,
-// which is here shown as valid account.
 AccountNumberCheck::Result method_B2(int *account, int *weight) {
     if (account[0] < 8) {
-	number2Array("2987654320", weight);
-	return algo01(11, weight, false, 10, account);
+	return method_02(account,weight);	// variant 1
     } // variant 1 (method "02")
     if (account[0] == 8 || account[0] == 9) {
-	number2Array("2121212120", weight);
-	return algo01(10, weight, true, 10, account);	
+	return method_00(account,weight);	// variant 2
     } // variant 2 (method "00")
     return AccountNumberCheck::ERROR; // should never happen, to be on the safe side
 }
@@ -1254,7 +1277,7 @@ AccountNumberCheck::Result method_B4(int *account, int *weight) {
 	return method_00(account,weight);	// variant 1
     if (account[0] < 9) {
 	number2Array("0987654320", weight); weight[0] = 10;
-	return algo01(11, weight, false, 10, account);
+	return algo02(11, weight, 10, account, 0, 8);
     } // variant 2 (method "02" using modified weight)
     return AccountNumberCheck::ERROR; // should never happen, to be on the safe side
 }
@@ -1342,6 +1365,22 @@ AccountNumberCheck::Result method_C2(int *account, int *weight) {
     return method_00(account,weight);	// variant 2
 }
 
+// new method C3 and C4 valid from March 5th, 2007, checked with Bundesbank-Testnumbers
+AccountNumberCheck::Result method_C3(int *account, int *weight) {
+    if (account[0] != 9) {
+	return method_00(account,weight);       // variant 1
+    } else {
+	return method_58(account,weight);	// variant 2
+    }
+}
+AccountNumberCheck::Result method_C4(int *account, int *weight) {
+    if (account[0] != 9) {
+	return method_15(account,weight);       // variant 1
+    } else {
+	return method_58(account,weight);	// variant 2
+    }
+}
+
 struct method_func_s {
     const char *str;
     AccountNumberCheck::MethodFunc func;
@@ -1464,6 +1503,8 @@ const struct method_func_s cb_funcs[] = {
   { "B9", method_B9},
   { "C1", method_C1},
   { "C2", method_C2},
+  { "C3", method_C3},
+  { "C4", method_C4},
   { 0, 0} // Important: The array has to end with the {0,0} entry,
 	  // otherwise initMethodMap() will runaway and crash!
 };
